# Default was 800x600

Write-Host "`n  Battlefield Vietnam Menu Resolution Changer by BFV.ovh" -ForegroundColor green
Write-Host "  Visit https://www.bfmods.com/viewtopic.php?f=31&t=1653`n" -ForegroundColor white
Write-Host "This app scans BfVietnam.exe in the " -NoNewline
Write-Host "local directory" -ForegroundColor white -NoNewline
Write-Host ", and it"
Write-Host "removes the 800x600 hard-coded resolution for the main menu`n"
Write-Host "Before continuing, copy your " -NoNewline
Write-Host "BfVietnam.exe" -ForegroundColor white -NoNewline
Write-Host " executable into"
Write-Host "this app's folder and then input your desired resolution, a"
Write-Host "new executable will be created: move it back to game folder`n"

# Changing and publishing this script without mentioning bfv.ovh is not allowed
# Original idea comes from http://www.bfmods.com/viewtopic.php?f=31&t=1653

Write-Host "New main menu resolution " -NoNewline
Write-Host "width" -ForegroundColor white -NoNewline
Write-Host ": " -NoNewline
[uint32]$Width = Read-Host
Write-Host "New main menu resolution " -NoNewline
Write-Host "height" -ForegroundColor white -NoNewline
Write-Host ": " -NoNewline
[uint32]$Height = Read-Host

Write-Host ""
Write-Host "Setting main menu resolution to: ${Width}x${Height}`n"

# Note: Codepage 28591 returns a 1-to-1 char to byte mapping
$Encoding = [Text.Encoding]::GetEncoding(28591)

function fileToString($path) {
    $Stream = New-Object IO.FileStream -ArgumentList (Resolve-Path $path), 'Open', 'Read'    
    $StreamReader = New-Object IO.StreamReader -ArgumentList $Stream, $Encoding
    $BinaryText = $StreamReader.ReadToEnd()
    $StreamReader.Close()
    $Stream.Close()
    return $BinaryText
}

function asBytesLE($lengthDecimal) {
    $lengthAsBytes = [BitConverter]::getBytes($lengthDecimal)
    # Convert to little-endian (switch order of bytes) if system is big-endian
    if (![BitConverter]::IsLittleEndian) {
        $lengthAsBytes = [Array]::reverse($lengthAsBytes)
    }
    return $lengthAsBytes;
}

function bytesAsString($bytes) {
    return $Encoding.GetString($bytes)
}

$WidthString = bytesAsString(asBytesLE($Width))
$HeightString = bytesAsString(asBytesLE($Height))
$BinaryString = fileToString("BfVietnam.exe")

# \xnn Matches an ASCII character, where nn is a two-digit hexadecimal character code.
$HotpatchableRegex = [Regex] '(\x20\x03\x00\x00)(?<keep>\xC7.{2,3})(\x58\x02\x00\x00)'
$matches = $HotpatchableRegex.Matches($BinaryString)

# First group (old width) is replaced with new width; second is kept the same; third group (old height) is replaced with new height
# Had to escape dollar with backticks, as it's regex matcher group, not powershell variable
# Had to use named group, otherwise heightString value as bytes might interfere (depending on what it is)
$Replacement = "${WidthString}`${keep}${HeightString}"

$Result = $HotpatchableRegex.Replace($BinaryString, $Replacement)
$ResultBytes = $Encoding.GetBytes($Result)
[System.IO.File]::WriteAllBytes("BfVietnam${Width}x${Height}.exe", $ResultBytes);

if ($?)
{Write-Host "Done! " -ForegroundColor green -NoNewline}
else
{Write-Host "Something went wrong"}

Read-Host "Press Enter to exit" | Out-Null